/********************************************************

         Windows ACM-based audio decoder
	 Copyright 2000 Eugene Smith (divx@euro.ru)
	 Last modified: 13.07.2000

*********************************************************/

#include <stdio.h>
#include "acm.h"
#include <string.h>
/*char cfmt[]={0x61, 0x01, 0x02, 0x00, 0x22, 0x56, 0x00, 0x00, 
0xa5, 0x0f, 0x00, 0x00, 0xba, 0x00, 0x10, 0x00,
0x0a, 0x00, 0x00, 0x04, 0x00, 0x00, 0x01, 0x00,
0xba, 0x00, 0x00, 0x00};
*/
ACM_Decoder::ACM_Decoder():m_status(0){}
int ACM_Decoder::Init(WAVEFORMATEX* pwf, void* extension)
{
    printf("Entering ACM_Decoder::Init\n");
    if(pwf==0)
	return -1;
    m_wstr=*pwf;
    if(extension==0)
	return -1;
/*    if(m_wstr.cbSize<10)
	return -1;*/
    memcpy(ext_info, pwf, sizeof(WAVEFORMATEX));
    memcpy((char*)ext_info+18, extension, min(32, (unsigned)m_wstr.cbSize));
    if(m_wstr.nSamplesPerSec==0)
	return -1;
	
//    MSACM_RegisterAllDrivers();
    
//    WAVEFORMATEX wf;
    wf.nChannels=m_wstr.nChannels;
    wf.nSamplesPerSec=m_wstr.nSamplesPerSec;
    wf.nAvgBytesPerSec=2*wf.nSamplesPerSec*wf.nChannels;
    wf.wFormatTag=WAVE_FORMAT_PCM;
    wf.nBlockAlign=2*m_wstr.nChannels;
    wf.wBitsPerSample=16;
    wf.cbSize=0;
/*    
    if(memcmp(ext_info, cfmt, sizeof(cfmt))!=0)
    {
	printf("WARNING: ext_info incorrect\n");
    }	
*/
    
    HRESULT hr=acmStreamOpen(
		&srcstream,    
		(HACMDRIVER)NULL,       
		(WAVEFORMATEX*)ext_info,  
		(WAVEFORMATEX*)&wf,  
		NULL,  
		0,     
		0,     
		0);
//		ACM_STREAMOPENF_NONREALTIME );
	
    if(hr!=S_OK)
    {
    	if(hr==ACMERR_NOTPOSSIBLE)
    	{
    		printf("Unappropriate audio format\n\n");
    		goto error;
	}
	printf("ACM_Decoder: acmStreamOpen error %X\n\n", hr);
	goto error;
    }
    m_status=1;
    return 0;
error:
    return -1;
}

int ACM_Decoder::Convert(unsigned char* in_data, unsigned in_size,
	     unsigned char* out_data, unsigned out_size,
	    unsigned* size_read, unsigned* size_written)
{
    HRESULT hr;
    
    if(m_status!=1)
    {
	printf("Bad status\n");
	goto fail;
    }	
    if(!in_data)
	goto fail;
    if(!out_data)
	goto fail;
	
    DWORD srcsize;
//    printf("ACM_Decoder: data starts with %X %X %X\n", 
//        *(int*)in_data, *(int*)(in_data+4),  *(int*)(in_data+8));
    
//    printf("ACM_Decoder: received request to convert %d bytes to %d bytes\n",
//	    in_size, out_size);
//    acmStreamSize(srcstream, in_size, &srcsize, ACM_STREAMSIZEF_SOURCE);
//    printf("%d\n", srcsize);
    acmStreamSize(srcstream, out_size, &srcsize, ACM_STREAMSIZEF_DESTINATION);
//    printf("%d\n", srcsize);
    ACMSTREAMHEADER ash;
    memset(&ash, 0, sizeof(ash));
    ash.cbStruct=sizeof(ash);
    ash.fdwStatus=0;
    ash.dwUser=0; 
    ash.pbSrc=in_data; 
    ash.cbSrcLength=srcsize; 
    ash.pbDst=out_data; 
    ash.cbDstLength=out_size; 
  // 
printf("ACM_Decoder: src: %X %X %X\n", ash.cbStruct, srcsize, out_size); 
    hr=acmStreamPrepareHeader(
      srcstream,          
      &ash,  
    	0);
    if(hr!=S_OK)
    {
    	printf("ACM_Decoder: acmStreamPrepareHeader error %X\n\n", hr);
    	goto fail;
    }
    

    hr=acmStreamConvert(
    	  srcstream,          
	  &ash,  
    	  0
    	);
    if(hr!=S_OK)
    {
	printf("ACM_Decoder: acmStreamConvert error %X\n", hr);
	goto fail;
    }
/*    if(qw==0)
    {
	FILE* qz=fopen("./test.qqq", "wb");
	fwrite(out_data, 1, out_size, qz);
	fclose(qz);
	qw=1;
    }		*/

//    printf("Uncompressed %d bytes from %d\n", ash.cbDstLengthUsed, ash.cbSrcLengthUsed);

	
    if(size_read)*size_read=ash.cbSrcLengthUsed;
    if(size_written)*size_written=ash.cbDstLengthUsed;

    hr=acmStreamUnprepareHeader(
    	srcstream,          
    	&ash,  
    	0);
    return 0;
fail:		
    if(size_read)*size_read=0;
    in_size=0;
    return -1;
}

int ACM_Decoder::Close()
{
    if(m_status==0)
	return 0;
    acmStreamClose(srcstream, 0);
    return 0;
}    