/********************************************************

         Audio decoder interface
	 Copyright 2000 Eugene Smith (divx@euro.ru)
	 Last modified: 01.08.2000

*********************************************************/
	 



#include "gsm61/gsm61dec.h"
#include "mpeg/mpegsound.h"
#include "pcm/pcm.h"
#include "acm/acm.h"
#include "adpcm/adpcm.h"
#include <audiodecoder.h>
//very strange, but if I move this constructor into .h,
//I receive strange errors... Compilers of this file and avifile/AviFile.cpp
//somehow disagree about size of WAVEFORMATEX.
AudioDecoder::AudioDecoder()
:decoder(0), m_status(0){}
int AudioDecoder::Init(WAVEFORMATEX* format, void* extension)
{
    if(format==NULL)
	return -1;
    if(m_status>0)
	return -1;
    switch(format->wFormatTag)
    {
	case 0x01://PCM
        decoder=new PCM_Decoder;
	    printf("Audio in PCM format\n");
	    break;
	case 0x2://MS ADPCM
	    decoder=new ACM_Decoder;
	    printf("Audio in MS ADPCM format\n");
	    break;
	case 0x11://IMA ADPCM
	decoder=new ADPCM_Decoder;
	    printf("Audio in ADPCM format\n");
	    break;
	case 0x31://MS GSM 6.10
        case 0x32://MSN Audio
	    decoder=new MSGSM_Decoder;
	    printf("Audio in MS GSM 6.10 format\n");
	    break;
	case 0x50://MPEG Layer-1,2    
	    decoder=new Mpegtoraw;
	    printf("Audio in MPEG Layer-1/2 format\n");
	    break;
	case 0x55://MPEG Layer-3
	    decoder=new Mpegtoraw;
	    printf("Audio in MPEG Layer-3 format\n");
	    break;
	case 0x160:
	case 0x161://DivX audio
	    decoder=new ACM_Decoder;
	    printf("Audio in DivX WMA format\n");
	    break;
	default:
	    printf("Audio format ID %d unsupported\n", format->wFormatTag);
	    break;
    }
    in_fmt=*format;
    if(decoder)
    {
	printf("Calling decoder->Init()\n");
	if(decoder->Init(format, extension)==0)
	{
	    m_status=1;
    	    return 0;
	}    
	if(format->wFormatTag==0x2)
// Of course, I have msadp32.acm, but I don't think it's legal to redistribute it
// because it is only available as part of Windows.

// Another option is to port MS ADPCM decoder from xanim, but I start feeling myself
// sick after 5 minutes of dealing with xanim sources :(( 

// XA_ADecode_ADPCMM_PCM2M(snd_hdr,obuf,ocnt,buff_size) 
// XA_SND *snd_hdr; xaUBYTE *obuf;
// ooh...
	    printf("Sorry, MS ADPCM format is not supported. Copy manually msadp32.acm from your"
		       "Windows/System directory ( if you have any ) into /usr/lib/win32 to resolve"
		       "this problem.\n"); 
    }
    return -1;
}
int AudioDecoder::GetOutputFormat(WAVEFORMATEX* destfmt)
{
    if(destfmt==0)return -1;
    if(m_status==0)return -1;
//    return decoder->GetOutFormat(destfmt);
    WAVEFORMATEX* srcfmt=&in_fmt;
    if(in_fmt.wBitsPerSample==0)
	in_fmt.wBitsPerSample=16;
    if(in_fmt.wBitsPerSample==4)//ADPCM
	in_fmt.wBitsPerSample=16;
    *destfmt=in_fmt;
    destfmt->wFormatTag=1;//PCM
    destfmt->nAvgBytesPerSec=destfmt->nSamplesPerSec*destfmt->wBitsPerSample*destfmt->nChannels;
    destfmt->cbSize=0;
    printf("src [wFormatTag=%u, nChannels=%u, nSamplesPerSec=%lu, nAvgBytesPerSec=%lu, nBlockAlign=%u, wBitsPerSample=%u, cbSize=%u]\n", 
	  srcfmt->wFormatTag, srcfmt->nChannels, srcfmt->nSamplesPerSec, srcfmt->nAvgBytesPerSec, 
	  srcfmt->nBlockAlign, srcfmt->wBitsPerSample, srcfmt->cbSize);

    printf("dst [wFormatTag=%u, nChannels=%u, nSamplesPerSec=%lu, nAvgBytesPerSec=%lu, nBlockAlign=%u, wBitsPerSample=%u, cbSize=%u]\n", 
	  destfmt->wFormatTag, destfmt->nChannels, destfmt->nSamplesPerSec, destfmt->nAvgBytesPerSec, 
	  destfmt->nBlockAlign, destfmt->wBitsPerSample, destfmt->cbSize);
    return 0;
}    

int AudioDecoder::Convert(unsigned char* in_data, unsigned in_size,
	     unsigned char* out_data, unsigned out_size,
	    unsigned* size_read, unsigned* size_written)
{
    if(m_status==0)
	return 0;
    return decoder->Convert(in_data, in_size, out_data, out_size,
	    size_read, size_written);
}
int AudioDecoder::Close()
{
    if(m_status==0)
	return -1;
    if(decoder)
    {
	decoder->Close();
	delete decoder;
	decoder=0;
	m_status=0;
	return 0;
    }
    return -1;
}

  	      