/* AVI codec */

/*  XMPS - X MPEG Player System
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "avi_codec.h"

extern "C" {

#include "avi_configure.h"
#include "loader.h"
XMPSCodec avi_codec = {

  NULL,
  NULL,
  "AVI WIN32 Codecs",
  1,
  
  avi_init,
  avi_set_surface,
  avi_set_callback,
  avi_scale,
  avi_move_into_display,
  avi_get_available_formats,
  avi_set_format,
  avi_set_lock_func,
  avi_set_mode,
  
  avi_is_our_file,
  avi_get_status,
  avi_has_file,
  avi_video_info,
  avi_get_actual_position,
  avi_get_file_size,
  avi_get_volume,
  avi_set_volume,
  
  avi_open,
  avi_play,
  avi_stop,
  avi_pause,
  avi_rewind,
  avi_seek,
  avi_close,
  
  avi_about,
  avi_configure,
  avi_load_options,
  avi_save_options,
  avi_file_info_box,
};

  GList                 *avi_format_list;
  XMPS_VideoLockFunc     avi_lock_func;
  XMPSVideoMode          avi_mode;
  XMPSVideoFormat        avi_format;
  static XMPSCodecStatus avi_status;
  XMPS_DisplayCallback   avi_xmps_callback;
  static IAviPlayer*     player;
  char                  *avi_filename;
  void			*avi_surface=0;

XMPSCodec *get_codec_info(void)
{
  return &avi_codec;
}

void avi_init(void)
{
  GList           *node;
  XMPSVideoFormat *f;

  avi_status = CODEC_STOPPED;

  avi_format_list = NULL;

  node = (GList *) malloc(sizeof(GList));
  
  f = (XMPSVideoFormat *) malloc(sizeof(XMPSVideoFormat));
  *f = XMPS_VIDEO_RGB16;

  node->data = f;
  node->next = NULL;

  avi_format_list = node;  
}

static void drawfunc(char* data)
{
    if(player==0)
      return;
    if(avi_surface && data)
      memcpy(avi_surface, data, 2*player->width()*player->height());
    if(avi_xmps_callback && data)
      avi_xmps_callback((byte *) data, NULL, 0, 0, player->width(), player->height());
//    else
//      DEBUG("avi","image pointer is NULL!",NULL);
}

static void killhandler(int result)
{

}

static double lastpos=0;

static void posfunc(double pos)
{
  lastpos = pos;
}


int avi_open(char *filename)
{
    chdir(codec_path);
    
    if( player )
	player->endPlayer();
    else
	player = CreateAviPlayer();
    try
    {
        player->initPlayer(filename, 16);
    }
    catch(FatalError& e)
    {
	e.Print();
        return -1;
    }	
    avi_filename = filename;

    player->setDrawCallback(&drawfunc);
    player->setKillHandler(&killhandler);
    player->start();
    player->pause(1);
    
    return 1;
}

void avi_play()
{
    if(!player)return;    
  if(!player->isValid())
    {
    int result;
    try
    {	
      result = player->initPlayer(avi_filename, 16);
    }
    catch(FatalError& e)
    {
	e.Print();//you can get error text from e and show message box here  
    }
      player->setDrawCallback(&drawfunc);
      
      player->setKillHandler(&killhandler);
      player->start();
      player->pause(1);
    }

  player->play();
  avi_status = CODEC_PLAYING;
}

void avi_about(void)
{

}

void            avi_configure(void)
{
  create_avi_config();
  show_avi_config();
}

void            avi_load_options(ConfigFile *cfg)
{
  codec_path = (char *) malloc(256);
  codec_path = "/home/dev/chavarri";
 
  if( cfg )
    {
      xmps_cfg_read_string(cfg, "DivX Codec", "codec_path", &codec_path);
      SetCodecPath(codec_path);
    }
}

void            avi_save_options(ConfigFile *cfg)
{
  xmps_cfg_write_string(cfg, "DivX Codec", "codec_path", codec_path);
}

int             avi_is_our_file(char *filename)
{
  return (strstr(filename, ".avi") || strstr(filename, ".AVI") 
      || strstr(filename, ".asf") || strstr(filename, ".ASF"));
}

void            avi_stop(void)
{
  if(player==0)
    return;
  player->stop();
  avi_status = CODEC_STOPPED;
}

void            avi_pause(void)
{
  if(player == 0)
      return;
  player->pause(1);
  avi_status = CODEC_STOPPED;
}

void            avi_rewind(void)
{
  //avi_seek(0); 
}

void            avi_seek(int percent)
{
  if(player == 0)
    return;
  player->reseek_exact(player->GetVideoLength()*percent/100.);
}

void            avi_get_volume()
{
  
}

void            avi_set_volume(int volume)
{

}

void            avi_file_info_box(char *filename)
{

}

void            avi_close(void)
{
  if(player==0)return;
  player->endPlayer();
  delete player;
  player=0;
}

void avi_set_surface (void *surface)
{
    if(surface==0)
	return;
    avi_surface=surface;
}

void avi_set_callback (XMPS_DisplayCallback callback)
{
  avi_xmps_callback = callback;
}

XMPSCodecStatus avi_get_status(void)
{
  return avi_status;
}

int  avi_has_file(void)
{
  return (player!=0);
}

void            avi_video_info(XMPSVideoInfo *info)
{
  if( player != 0 && info != 0 )
    {
       info->width  = player->width();
       info->height = player->height();
    }
}

void avi_scale(int x, int y)
{
  
}

unsigned int avi_get_actual_position(void)
{
  if( player != 0 )
    return (unsigned int) player->GetPos()*100/player->GetVideoLength();
  else
    return 0;
}

unsigned int avi_get_file_size(char *filename)
{
  FILE* file = fopen(filename, "r");
  fseek(file, 0, SEEK_END);
  int file_size = ftell(file);
  return file_size;
}

int avi_move_into_display(int x, int y)
{
  return 0;
}

GList* avi_get_available_formats(void)
{
  return avi_format_list;
}

int             avi_set_format(XMPSVideoFormat format)
{
  avi_format = format;
}

int             avi_set_lock_func(XMPS_VideoLockFunc f)
{
  avi_lock_func = f;
}

int             avi_set_mode(XMPSVideoMode mode)
{
  avi_mode = mode;
}

}; // extern

