/********************************************************

	Classes for reading AVIs
	Copyright 2000 Eugene Smith (divx@euro.ru)
	Last modified: 30.07.2000

*********************************************************/


#include "AviRead.h"
#include <iostream>
using namespace std;    
#define __MODULE__ "AviRead"
#ifndef TIMING
#define Debug if(0)
#else
#define Debug
#endif

IAviReadStream::~IAviReadStream()
{
}


IAviReadFile::~IAviReadFile()
{
}

/********************************************************************
 
 AviReadStream functions that are common for video and audio streams

*********************************************************************/
AviReadStream::~AviReadStream()
{
}

HRESULT AviReadStream::Eof() const
{
//    cout<<m_sample<<" "<<m_table_size<<endl;
#ifdef __FreeBSD__
    if (m_header.fccType == streamtypeAUDIO)
	return (m_sample >= m_header.dwLength);
#endif

    return (m_sample>=m_table_size);
}
HRESULT AviReadStream::Init(unsigned id, IvAVIReadStream* stream)
{
    if(stream==0)
	return -1;
    m_pIStream=stream;
    m_id=id;
    
    m_pIStream->Header(&m_header);
    m_pIStream->FormatSize(0, &m_format_size);
    m_format=new char[m_format_size];
    m_pIStream->ReadFormat(0, m_format, &m_format_size);

    m_table_size=stream->Length();	
    cout<<"Successfully initialized stream "<<id<<endl;
    cout<<"Chunk table size "<<m_table_size<<", format size "<<m_format_size<<endl;
    return 0;   
}


AviReadStream::StreamType AviReadStream::GetType() const
{
    if(m_header.fccType==streamtypeVIDEO)
	return Video;
	else
        if(m_header.fccType==streamtypeAUDIO)
	    return Audio;
	    else
	    return Other;
}
HRESULT AviReadStream::Seek(unsigned pos)
{
    cout<<"AviReadStream::Seek"<<endl;
    m_sample=pos;
    rem_size=0;
    if(pos>=GetEndPos())return -1;
    m_pIStream->ClearCache();
    return 0;
}
double AviReadStreamV::SeekToTime(double time)
{
    cout<<"AviReadStreamV::SeekToTime"<<endl;
    unsigned pos=(unsigned)(time*(double)m_header.dwRate/m_header.dwScale);
    if(pos>=m_table_size)
    {
        Seek(pos);
        return time;
    }    
    pos=m_pIStream->NearestKeyFrame(pos);
    Seek(pos);
    double qq=pos*(double)m_header.dwScale/m_header.dwRate;	
    return qq;
}
unsigned AviReadStream::GetNextKeyFrame() const
{
    if(m_pIStream==0)return 0;
    return m_pIStream->NextKeyFrame(m_sample);
}    
unsigned AviReadStream::GetPrevKeyFrame() const
{
    if(m_pIStream==0)return 0;
    return m_pIStream->PrevKeyFrame(m_sample-2);
}
int AviReadStream::ToNextKeyFrame()
{
    if(m_pIStream==0)return -1;
    unsigned newpos=m_pIStream->NextKeyFrame(m_sample);
    if(newpos==-1)return -1;
    m_sample=newpos;
    return 0;
}
int AviReadStream::ToPrevKeyFrame()
{
    if(m_pIStream==0)return -1;
    unsigned newpos=m_pIStream->PrevKeyFrame(m_sample-2);
    if(newpos==-1)return -1;
    m_sample=newpos;
    return 0;
}
    

HRESULT AviReadStream::ReadDirect(char* buffer, unsigned bufsize, unsigned samples,
	unsigned& samples_read, unsigned& bytes_read)
{
    if(m_pIStream==0)return -1;
    HRESULT hr=m_pIStream->Read(m_sample, samples, buffer, bufsize, (long*)&bytes_read, (long*)&samples_read);
    m_sample+=samples_read;
    return hr;
}
HRESULT AviReadStream::GetFrameFlags(int* flags)
{
    if(flags==0)return -1;
//    if(m_pTable==0)return -1;
    if(m_pIStream)return 0;
#warning FIXME
//    if(m_sample>m_table_size)return -1;
    *flags=(m_pIStream->IsKeyFrame(m_sample)?AVIIF_KEYFRAME:0);
//    int size=m_pTable[m_sample].size;
//    *flags=(size&0x80000000)?AVIIF_KEYFRAME:0;
    return 0;
}
double AviReadStream::GetTime() const
{
    return GetPos()*(double)m_header.dwScale/m_header.dwRate;
}
unsigned AviReadStream::GetPos() const
{
    return m_sample;
}

double AviReadStream::GetEndTime() const
{
    return m_header.dwLength*(double)m_header.dwScale/m_header.dwRate;
}
int AviReadStream::IsStreaming()
{
    return m_streaming;
}    
/******************************************************************

    These functions are meaningless for 'unknown type' stream
    
******************************************************************/    

HRESULT AviReadStream::GetVideoFormatInfo(void* bi)
{
    cerr<<"WARNING: GetVideoFormatInfo() for non-video stream"<<endl;
    return -1;
}    
HRESULT AviReadStream::GetAudioFormatInfo(void* bi, char** ext)
{
    cerr<<"WARNING: GetAudioFormatInfo() for non-audio stream"<<endl;
    return -1;
}  
    
HRESULT AviReadStream::SetBitDepth(unsigned depth)
{
    cerr<<"WARNING: Setting bit depth for non-video stream"<<endl;
    return -1;
}    
HRESULT AviReadStream::ReadFrame()
{
    cerr<<"WARNING: AviReadStream::ReadFrame() called for non-video stream"<<endl;
    return -1;
}

char* AviReadStream::GetFrame()
{
    cerr<<"WARNING: AviReadStream::GetFrame() called for non-video stream"<<endl;
    return NULL;
}

int AviReadStream::GetFrameSize()
{
    cerr<<"WARNING: AviReadStream::GetFrameSize() called for non-video stream"<<endl;
    return 0;
} 
HRESULT AviReadStream::SetOutputFormat()
{
    cerr<<"WARNING: AviReadStream::SetOutputFormat() called for non-video stream"<<endl;
    return -1; 
}

/*************************************************************

        AviReadStreamV ( video AVI stream ) functions
	
*************************************************************/	

HRESULT AviReadStreamV::GetVideoFormatInfo(void* bi)
{
    if(bi==0)
        return E_ERROR;
    memcpy(bi, m_format, m_format_size);
    return 0;
}    
 

HRESULT AviReadStreamV::StartStreaming()
{
    int result;
    if(m_streaming)return 0;
	videodecoder=new VideoDecoder;
	try
	{
	    videodecoder->Init(m_format);
	    videodecoder->Start();	
	    m_streaming=1;
	}
	catch(FatalError& error)
	{
	    error.Print();
	    delete videodecoder;
	    videodecoder=0;
	    return -1;
	}    
	return 0;
}

HRESULT AviReadStreamV::StopStreaming()
{
    int result;
    if(m_streaming==0)
    return 0;
    if(videodecoder)
    {
        delete videodecoder;
        videodecoder=0;
    }    
}
HRESULT AviReadStreamV::SetBitDepth(unsigned depth)
{
    if(videodecoder==0)return -1;
    videodecoder->Stop();
    HRESULT hr=videodecoder->SetBitDepth(depth);
    m_obh=videodecoder->DestFmt();
    videodecoder->Start();
    return hr;
}
HRESULT AviReadStreamV::GetOutputFormat(void* format, unsigned size)
{
	if(size<sizeof(BITMAPINFOHEADER))
	    return -1;
	if(format==0)
	    return -1;
	if(videodecoder==0)
	    return -1;
	*(BITMAPINFOHEADER*)format=videodecoder->DestFmt();
	return 0;
}

HRESULT AviReadStreamV::SetOutputFormat()
{
	if(videodecoder==0)return -1;
        m_obh=videodecoder->DestFmt();
#warning FIXME
	return -1; //not implemented

}

HRESULT AviReadStreamV::ReadFrame()
{
    if(m_pIStream==0)return -1;
    long lBytes, lSamples;
    long size;
//    cout<<"Current pos: "<<m_sample<<endl;
    if(GetType()!=Video)return -1;
    HRESULT hr=m_pIStream->Read(m_sample, 1, 0, 0, &size, 0);
    if(hr!=0)
    {
//	cout<<"WARNING: Read() failed"<<endl;
	return -1;
    }	
    if(size<0)
    {
//	cout<<"WARNING: size="<<size<<endl;
	return -1;
    }	
    char* temp_buffer=new char[size];
    hr=m_pIStream->Read(m_sample, 1, temp_buffer, size, &lBytes, &lSamples);
 
    if(lSamples!=1)
    {
//	cout<<"Read "<<lSamples<<" samples"<<endl;
	delete temp_buffer;
	return -1;
    }	
    unsigned in_size=0;
    unsigned ocnt=0;
    if(videodecoder)
        hr=videodecoder->DecodeFrame(temp_buffer, lBytes, m_pIStream->IsKeyFrame(m_sample)?AVIIF_KEYFRAME:0);
    else
        hr=-1;	
    delete temp_buffer;
    m_sample++;
    return hr;
}

char* AviReadStreamV::GetFrame()
{
    if(videodecoder==0)return NULL;
    return videodecoder->GetFrame();
}
int AviReadStreamV::GetFrameSize()
{
    return m_obh.biSizeImage;
}

HRESULT AviReadStreamV::ReadFrames(char* buffer, unsigned bufsize, unsigned samples,
    unsigned& samples_read, unsigned& bytes_read)
{
    long _samp=samples;
    long lBytes, lSamples;
    long size;
    long bufsize_avail=bufsize;
    char* temp_buffer=0;
    HRESULT hr=0;
    if(m_pIStream==0)return -1;
    if(samples==0)return -1;
    samples_read=0;
    bytes_read=0;
    while(_samp>0)
    {
        if(hr!=0)return -1;
	hr=m_pIStream->Read(m_sample, 1, 0, 0, &size, 0);
    	if(size<0)return -1;
	temp_buffer=new char[size];
	if(temp_buffer==NULL)return -1;
	hr=m_pIStream->Read(m_sample, 1, temp_buffer, size, &lBytes, &lSamples);
	_samp-=lSamples;
        unsigned in_size=0;
	unsigned ocnt=0;
        unsigned local_read;
	if(GetFrameSize()>bufsize)
	{
	    hr=-2;
	    goto fail;
	}        
	hr=videodecoder->DecodeFrame(temp_buffer, lBytes, m_pIStream->IsKeyFrame(m_sample)?AVIIF_KEYFRAME:0);
	if(hr!=0)
	{		
	    if(temp_buffer)delete temp_buffer;
	    return hr;
	}
	memcpy(buffer, videodecoder->GetFrame(), GetFrameSize());
	buffer+=GetFrameSize();

	bufsize-=GetFrameSize();
	bytes_read+=GetFrameSize();
	m_sample+=lSamples;
	samples_read+=lSamples;
    }	
fail:
    if(temp_buffer)delete temp_buffer;
    return hr;
}

/*************************************************************

	AviReadStreamA ( audio AVI stream ) Functions 

**************************************************************/

double AviReadStreamA::SeekToTime(double time)
{
    cout<<"AviReadStreamA::SeekToTime"<<endl;
    if(time<0)
	return -1;
    if(Seek(time*(double)m_header.dwRate/m_header.dwScale)!=0)
        return -1;
    return time;
}
const int wfmtx_size=18;//sizeof(...) depends on packing

HRESULT AviReadStreamA::GetAudioFormatInfo(void* bi, char** ext)
{
    if(bi==0)
        return E_ERROR;
    memcpy(bi, m_format, wfmtx_size);
    if(ext==0)
        return 0;
    if(((LPWAVEFORMATEX)bi)->cbSize>0)
    {
	if(m_format_size<(wfmtx_size+((LPWAVEFORMATEX)bi)->cbSize))
	{
	    cout<<"WARNING: GetAudioFormatInfo: bad audio structure"<<endl;
	    *ext=0;
	    return E_ERROR;
	}    
	*ext=new char[((LPWAVEFORMATEX)bi)->cbSize];
	memcpy(ext, m_format+wfmtx_size, ((LPWAVEFORMATEX)bi)->cbSize);
	return 0;
    }	
    else
    *ext=0;
    return 0;
}

unsigned AviReadStreamA::GetPos() const
{
    Debug cout<<"Sample "<<m_sample<<", rem_size"<<rem_size<<", sample_size "<<m_header.dwSampleSize<<endl;
    if((m_header.dwSampleSize==0))
	return m_sample;
    else
	return m_sample-rem_size/m_header.dwSampleSize;
}    

HRESULT AviReadStreamA::StartStreaming()
{
    int result;
    if(m_streaming)return 0;
	audiodecoder=new AudioDecoder;
	result=audiodecoder->Init((WAVEFORMATEX*)m_format, (m_format_size>wfmtx_size)?m_format+wfmtx_size:0);
	if(result!=0)
	{
	    cout<<"Failed to initialize audio decoder for format "<<((WAVEFORMATEX*)m_format)->wFormatTag<<endl;
	    delete audiodecoder;
	    audiodecoder=0;
	    return -1;
	}    
	m_streaming=1;
	rem_size=0;
	return 0;


}
HRESULT AviReadStreamA::StopStreaming()
{
    int result;
    if(m_streaming==0)
    return 0;
    if(audiodecoder)
    {
        audiodecoder->Close();
        delete audiodecoder;
        audiodecoder=0;
    }    
    if(m_header.dwSampleSize)
        if(rem_size/m_header.dwSampleSize<m_sample)
    	    m_sample-=rem_size/m_header.dwSampleSize;
    rem_size=0;
}

HRESULT AviReadStreamA::GetOutputFormat(void* format, unsigned size)
{
	if(size<sizeof(WAVEFORMATEX))
	    return -1;	
	if(format==0)
	    return -1;
	if(audiodecoder==0)
	    return -1;
	return audiodecoder->GetOutputFormat((WAVEFORMATEX*)format);
}
HRESULT AviReadStreamA::ReadFrames(char* buffer, unsigned bufsize, unsigned samples,
    unsigned& samples_read, unsigned& bytes_read)
{
    long lBytes, lSamples;
    long size;
    unsigned in_size=0;
    unsigned ocnt=0;
    unsigned local_read;
    char* temp_buffer=0;
    HRESULT hr=0;
    if(m_pIStream==0)return -1;
    if(samples==0)return -1;
    samples_read=0;
    bytes_read=0;

    temp_buffer=new char[bufsize];
    int rem_diff=min((long)rem_size, (long)bufsize);
    memcpy(temp_buffer, rem_buffer, rem_diff);
    if(rem_diff<rem_size)memcpy(rem_buffer, rem_buffer+rem_diff, rem_size-rem_diff);
    lBytes=0;
    if(bufsize-rem_diff)
    {
        hr=m_pIStream->Read(m_sample, samples, temp_buffer+rem_diff, bufsize-rem_diff, &lBytes, &lSamples);
	Debug printf("AviReadStreamA: Read %d samples from pos %d\n", lSamples, m_sample);
        m_sample+=lSamples;
        samples_read+=lSamples;
//	printf("AviReadStreamA::ReadFrames: Read %d samples ( %d bytes ), %d from last call\n", lSamples, lBytes, rem_diff);
        if(lBytes==0)
        {
            delete temp_buffer;
            return -1;
        }
    }
    lBytes+=rem_diff;
    hr=audiodecoder->Convert((unsigned char*)temp_buffer, lBytes, (unsigned char*)buffer, bufsize, 
        &local_read, &ocnt);
    rem_size=min(lBytes-local_read, (unsigned long)sizeof rem_buffer);
//    printf("AviReadStreamA::ReadFrames: converted %d bytes of %d to %d, bufsize %d\n", local_read, lBytes, ocnt, bufsize);
    memcpy(rem_buffer, temp_buffer+local_read, rem_size);
    bytes_read=ocnt;
fail:
    if(temp_buffer)delete temp_buffer;
    Debug printf("AviReadStreamA: new m_sample is %d\n", m_sample);
    return hr;
}

/************************************************************

		    AviReadFile functions
		    
************************************************************/		    

AviReadFile::AviReadFile(const char* name)
{
    try
    {
	m_streams.clear();
	m_pIFile=0;
	try
        {
	    m_pIFile=CreateAVIReadHandler(name);
        }
        catch(FatalError& e)
	{
	    e.Print();
        }	
        catch(...)
	{
	    cout<<"Some strange exception"<<endl;
        }	
	if(m_pIFile==0)
        {	
	    try
    	    {
    	        m_pIFile=CreateAVIReadHandler2(name,0,0);
	    }
    	    catch(FatalError& e)
            {	
	        e.Print();
	    }	
            catch(...)
	    {
		cout<<"Some strange exception"<<endl;
	    }	
	}
        if(m_pIFile==0)
	    throw FATAL("Can't open file");//never get here
        cout<<"Successfully opened"<<endl;
//  if(m_pIFile==0)return 0;
        IvAVIReadStream* stream;
        int i=0;
        m_vcount=m_acount=0;
        while(stream=m_pIFile->GetStream(mmioFOURCC('v', 'i', 'd', 's'), m_vcount++));
        m_vcount--;
        while(stream=m_pIFile->GetStream(mmioFOURCC('a', 'u', 'd', 's'), m_acount++));
        m_acount--;
	m_streams.clear();
//    m_streams=new AviReadStream[m_vcount+m_acount];
	for(i=0; i<m_vcount; i++)
        {
	    AviReadStream* str=new AviReadStreamV;
	    str->Init(i, m_pIFile->GetStream(mmioFOURCC('v', 'i', 'd', 's'), i));
	    m_streams.push_back(str);
        }	
	for(i=0; i<m_acount; i++)
        {
	    AviReadStream* str=new AviReadStreamA;
	    str->Init(m_vcount+i, m_pIFile->GetStream(mmioFOURCC('a', 'u', 'd', 's'), i));
    	    m_streams.push_back(str);
        }	
	m_pIFile->Header(&m_header);	
	cout<<"Successfully opened "<<name<<". "<<m_vcount<<" video streams, "<<m_acount<<" audio streams"<<endl;
        cout<<"Length "<<m_header.dwTotalFrames<<endl;
    }
    catch(...)
    {
        _destruct();
	throw;
    }
}	    
unsigned AviReadFile::StreamCount()
{
    return m_vcount+m_acount;
}	
unsigned AviReadFile::VideoStreamCount()
{
    return m_vcount;
}
unsigned AviReadFile::AudioStreamCount()
{
    return m_acount;
}	

IAviReadStream* AviReadFile::GetStream(unsigned stream_id, AviStream::StreamType type)
{
    if(type==AviStream::Video)
        if(stream_id<VideoStreamCount())
	    return m_streams[stream_id];
    if(type==AviStream::Audio)
	if(stream_id<AudioStreamCount())
	    return m_streams[m_vcount+stream_id];
    return NULL;
}
HRESULT AviReadFile::GetFileHeader(MainAVIHeader* header)
{
    if(header)*header=m_header;
    return 0;
}
void AviReadFile::_destruct()
{
    if(m_streams.size())
	for(int i=0; i<m_streams.size(); i++)
	    delete m_streams[i];
    m_streams.clear();
    delete m_pIFile;
    m_pIFile=0;
    return;
}    
AviReadFile::~AviReadFile()
{
    _destruct();
}





IAviReadFile* CreateIAviReadFile(const char* name)
{
    return new AviReadFile(name);
}    
