/********************************************************

	AVI player object
	Copyright 2000 Eugene Smith (divx@euro.ru)
	Last modified: 10.08.2000

*********************************************************/

#ifndef QUIET
#define debug_out(X) Debug cout<<X<<": "<<(t2-t1)/freq<<" ms"<<endl
#ifndef TIMING
#define Debug  if(0)
#else
#define Debug 
#endif /*TIMING*/
#else
#define debug_out(X)
#define Debug if(0)
#endif /*QUIET*/

#include <stdio.h>
#include <time.h>//time
#include <iostream.h>
#include <unistd.h>//usleep, close
#include <pthread.h>
#include <errno.h>//EDEADLK

#include <except.h>
#include "aviplay_impl.h"


#define __MODULE__ "IAviPlayer"

static double old_freq()
{
    int i=time(NULL);
    int x,y;
    while(i==time(NULL));
    x=localcount();
    i++;
    while(i==time(NULL));
    y=localcount();
    return (double)(y-x)/1000.;
}
CPU_Freq::CPU_Freq()
{
	FILE *f = fopen ("/proc/cpuinfo", "r");
	char line[200];
	char model[200]="unknown";
	char flags[500]="";
	char	*s,*value;
	
	freq=-1;
	if (!f)
	{
	    printf("Can't open /proc/cpuinfo for reading\n");
	    //old method of determining CPU frequency
	    freq=old_freq();
	    return;
	}    
	while (fgets(line,200,f)!=NULL) 
	{

		/* NOTE: the ':' is the only character we can rely on */
		if (!(value = strchr(line,':')))
			continue;
		/* terminate the valuename */
		*value++ = '\0';
		/* skip any leading spaces */
		while (*value==' ') value++;
		if ((s=strchr(value,'\n')))
			*s='\0';

		if (!strncasecmp(line, "cpu MHz",strlen("cpu MHz"))) 
		{
		    freq=atof(value);
		    freq*=1000;
		}
		if(!strncasecmp(line, "model name", strlen("model name")))
		    strncpy(model, value, sizeof(model)-1);
		if(!strncasecmp(line, "flags", strlen("flags")))
		    strncpy(flags, value, sizeof(flags)-1);
		continue;
		
	}
	if(freq<0)freq=old_freq();
        cout<<freq/1000.<<" MHz "<<model<<" processor detected"<<endl; 
        cout<<"Available flags: "<<flags<<endl;	
	if(strstr(flags, "tsc")==0)
        {
	    printf("ERROR: no time-stamp counter found! Quitting.\n");
	    exit(1);
	}
#ifdef MMX
	if(strstr(flags, "mmx")==0)
	{
	    printf("ERROR: using MMX binary on non-MMX system. Quitting.\n");
	    exit(1);
	}
#endif
	fclose(f);
}

CPU_Freq freq;
IAviPlayer::~IAviPlayer(){}
static long long longcount()
{
    unsigned int i;
    unsigned int j;
    __asm__ __volatile__("rdtsc\n\t":"=a"(i), "=d"(j):);
    return ((long long)j<<32)+(long long)i;
}

static inline float to_float(long long t2, long long t1)
{
    long long qw=t2-t1;
    qw=(long long)(qw/freq);//milliseconds;
    return ((int)qw)/1000.;
}

#define min(X,Y) ((X)<(Y)?(X):(Y))
#define max(X,Y) ((X)>(Y)?(X):(Y))
double AviPlayer::getAsync(int update)
{
	unsigned int t1,t2;
	t1=localcount();
	double stream_time;	
	if(time_start==0)
	{
	    Debug cout<<"AviPlayer::getAsync(): resetting"<<endl;
            time_start=longcount();
	    if(stream)
	        frame_start=stream->GetTime();
	    if(ar)
		ar->clear();
	}
	long long time_current=longcount();
	double actual_time=0;
//#ifndef USE_SDL
	if(ar && (!ar->Eof()))
	    actual_time=ar->getTime();
	else
//#endif
	    actual_time=frame_start+to_float(time_current, time_start);
	Debug cout<<"AviPlayer::getAsync(): actual_time "<<actual_time<<endl;
	if(stream)
	    stream_time=stream->GetTime();
	else
	    stream_time=actual_time;
	Debug cout<<"AviPlayer::getAsync(): stream_time "<<stream_time<<endl;
//	cout<<"Async "<<stream_time-actual_time<<" as "<<stream_time<<" - "<<actual_time<<endl;
	return stream_time-actual_time;
}

void* main_thread(void* arg) 
{
    AviPlayer& a=*(AviPlayer*)arg;
    IAviReadStream* stream=a.stream;
    int& initialized=a.initialized;
    int& quit=a.quit;
    KILLHANDLER& killhandler=a.killhandler;
    DRAWFUNC& drawfunc=a.drawfunc;
    
    unsigned int g1, g2;
    unsigned int t1, t2;
    
    while(1)
    {
	int& debug_status=a.debug_status;
	debug_status++;
	debug_status%=25;

	int novideo=0;
	int drop_frame=0;

	if(quit)
	{
#ifndef QUIET
	    cout<<"Quitting main thread"<<endl;
#endif	    
	    return 0;    
	}    
	if(a.hangup)
	{
	    initialized=0;
	    a.hangup=0;
	}    
	if(!initialized)
	{
#ifndef QUIET	
	    if(debug_status%10)cout<<"Waiting for start()"<<endl;
#endif	    
	    usleep(200000);
	    continue;
	}    

	double async=a.getAsync(1);
	Debug cout<<"main_thread(): 1) async() is "<<async<<endl;
	if(stream)
	    if(stream->Eof())
	    {
		initialized=0;
		if(a.ar)
		{
		    a.ar->stop();
		    a.ar->wake();
		}    
#ifndef QUIET		
		printf("Main thread waiting\n");
#endif		
		usleep(100000);
		continue;
	    }	

//	printf("Input:async %f ms\n", 1000.*async);

	if(async>.1)
	{
#ifndef USE_ESD	
	    if(a.ar)
	    {
		double frame_time=a.ar->buffer_time();
		Debug cout<<"main_thread(): frame_time "<<frame_time<<endl;
		// low-water mark for audio queue is 0.3 seconds.
		// high-water mark depends on bitrate; it's 80000 bytes + size of DMA buffer,
		// for my ESS1868 it's 145536 bytes or approx. 0.8 seconds for 44100/16/stereo.
		
		// We try to fill audio queue every pass of this loop, unless frame drop 
		// is so large that async becomes less -.5 ( what generally should
		// never happen ). In case that buffer_time() reaches high-water mark, 
		// doAudioExtract() just returns, without any audio decompression.
		// If buffer_time() falls below low-water mark ( it means that
		// our default decoding speed - 10 kbytes each video frame - 
		// is not enough ), this check fails and we start calling 
		// doAudioExtract() continuously.
		if(frame_time>.3)
		{
//			printf("Waiting for %f ms\n", 1000.*frame_time);
			usleep((int)min(1000000*(async-.1), 1000000*(frame_time-.3)));
	        }
	    }	
	    
	    else
#endif 
	    {
/*		async=a.getAsync(0);
		printf("1:async %f ms\n", 1000.*async);
		async=a.getAsync(0);
		printf("2:async %f ms\n", 1000.*async);
		async=a.getAsync(0);
		printf("3:async %f ms\n", 1000.*async);*/
//	    	printf("Waiting for %f ms\n", 1000.*(async-.1));
		if(async>.1)
		    usleep((int)(1000000*(async-.1)));
	    }	
	}	
	async=a.getAsync();
	Debug cout<<"main_thread(): 2) async() is "<<async<<endl;

//	printf("Async %f ms\n", 1000.*async);
	//
	//  If we've run out of audio data, let's put it into /dev/dsp
	//	as fast as we can.
	//
	if(async>=.1)
	    novideo=1;
	//
	//	otherwise,
	//
	if(novideo==0)
	//
	//	unless pic is older than audio for at least 100 ms,
	//	we will draw one frame.
	//
	if(async<=-.2)
	    drop_frame=1;
	else
	    if(async<=-.1)
		drop_frame=!(a.frames_video%3);//yes
	if(drop_frame)
	    Debug cout<<"main_thread(): dropping frame"<<endl;
	if(novideo)
	    Debug cout<<"main_thread(): Audio underrun, not drawing"<<endl;
//	Debug cout<<"async: "<<1000*async<<endl;
/*	    audio_buf_info zz;
	    ioctl(a.audio_fd, SNDCTL_DSP_GETOSPACE, &zz);
		
        if(a.debug_status%5==0)cout<<"Frame size: "<<a.frame_size<<" "<<(signed)Snd_Limit-zz.bytes<<endl;*/
	if(((drop_frame==0) && (novideo==0)) || a.paused)
	{
//	    printf("Reading frame\n");
    	    t1=localcount();
	    if(stream)
	    	    (*drawfunc)(stream->GetFrame());
    	    t2=localcount();
//	    debug_out("video_extract");
	    drop_frame=0;
    	}
	else
	{
	    if(drop_frame && (!a.paused))
	    {
#ifndef QUIET
		cout<<"!"<<flush;
		Debug cout<<endl;
#endif
		a.frame_drop++;
	    }
	    if(!a.paused)
		(*drawfunc)(NULL);
	}
        a.drop.Insert("Drop", drop_frame*100.);
	Debug cout<<"main_thread(): current video pos "<<stream->GetTime()<<endl;
        t1=localcount();
	if(a.ar)
	    if(!a.ar->Eof())
		if(!a.paused)
		    if(async>-.5)
			a.ar->doAudioExtract(stream->GetTime());
			
	t2=localcount();
//        debug_out("audio_extract");
	t1=localcount();
	if(stream)
    	    if((!stream->Eof()))
		if(novideo==0)
		    if(!a.paused)
		        if(a.stream->ReadFrame()==0)
			    a.frames_video++;
	t2=localcount();
//        debug_out("video_out");		 
        if(a.paused)
	{
	    if(a.stream)
    		(*drawfunc)(a.stream->GetFrame());
	    usleep(50000);
        }
    }//while(1)
}

AviPlayer::AviPlayer()
{
    stream=0;
    initialized=0;
    hangup=0;
    paused=0;
    quit=0;
    time_start=0;
    frame_start=0;
    frames_video=0;
    frame_drop=0;
    clip=0;
    ar=0;
    m_async=0;
}

AviPlayer::~AviPlayer()
{
    endPlayer();
}

int AviPlayer::initPlayer(const char* filename, int bitdepth, const char* subname) throw(FatalError)
{
    int result;
    try
    {
        if(clip==0)clip=CreateIAviReadFile(filename);

	cout<<"File "<<filename<<" successfully opened"<<endl;
        cout<<clip->StreamCount()<<" streams"<<endl;

	IAviReadStream* audiostream=clip->GetStream(0, IAviReadStream::Audio);

        if(audiostream==0)
	    cout<<"WARNING: File does not contain audio streams"<<endl;
	else
        {
	    if(ar)delete ar;
	    ar=0;
	    try
	    {
#ifdef USE_SDL	    
		ar=new SDL_AudioRenderer(audiostream);
		ar->setAsync(m_async);
#else
		ar=new AudioRenderer(audiostream);
#endif		
	    }
	    catch(FatalError& error)
    	    {
		error.Print();
	    }	
	}    		
    
        stream=clip->GetStream(0, IAviReadStream::Video);

	if(stream==0)
	    throw FATAL("File does not contain video streams");

	// I read subtitles here, so that I already know stream
	if(subname!=NULL)
    	    initSubtitles(subname);
	else	    
	if(strlen(filename)>=4)
    	    if(strncasecmp(filename+strlen(filename)-4, ".avi", 4)==0)
		{
		    char* subname=(char*)malloc(strlen(filename)+1);
		    strcpy(subname, filename);
		    strcpy(subname+strlen(filename)-4, ".sub");
		    if(initSubtitles(subname)==-1)
		    {
		    	strcpy(subname+strlen(filename)-4, ".SUB");
			initSubtitles(subname);
		    }
		    free(subname);
		}
    
        result=stream->StartStreaming();
    	if(result!=0)
    	    throw FATAL("Failed to initialize decoder object");
	result=stream->SetBitDepth(bitdepth);
        if(result!=0)
	    throw FATAL("Current bit depth is unsupported");

        int w=384,h=70;
	if(stream)
        {
	    stream->GetVideoFormatInfo(bitmapinfo);
	    w=*(int*)(bitmapinfo+4);
	    h=*(int*)(bitmapinfo+8);
        }

        m_width=w;
        m_height=h;
        frames_video=0;
        frame_drop=0;
        time_start=0;	
        quit=0;
	if(stream)
    	    stream->SeekToTime(0);
	short fs;
	__asm__ __volatile__ ("movw %%fs, %%ax":"=a"(fs));
	cout<<"Before pthread_create: fs is "<<fs<<endl;
	fs_seg=fs;
        if(stream)
	    pthread_create(&main_thread, NULL, ::main_thread, (void*)this);
        drop.AddField("Drop", 250);
        return 0;
    }
    catch(FatalError& error)
    {
	if(clip)
	    delete clip;
	clip=0;
	stream=0;
	throw FatalError(error);
    }
}
int AviPlayer::initSubtitles(const char* filename)
{
	double frame_time = 0.0;
	if (stream)
	{
	    frame_time = stream->GetFrameTime();
	    cout<<"Frame time "<<frame_time<<endl;
	}    
	FILE* file=fopen(filename, "rb");
	subtitles.clear();
	if(file==0)
	{
#ifndef QUIET	
		cout<<"File "<<filename<<" not found"<<endl;
#endif		
		return -1;
	}
	while(!feof(file))
	{
		subtitle_line line;
		if (line.read(file, frame_time) == 0)
		    subtitles.push_back(line);
		else
		    break;
	}
	fclose(file);
#ifndef QUIET	
	cout<<"Subtitles: read "<<subtitles.size()<<" lines"<<endl;
#endif	
	return 0;
}
const char* AviPlayer::GetCurrentSubtitles()
{
	if(stream==NULL)return "";
	if(subtitles.size()==0)return "";
	unsigned stream_pos=stream->GetPos();
	int ind_start=0, ind_end=subtitles.size()-1;
	if(stream_pos>subtitles[ind_end].end)return "";
	if(stream_pos<subtitles[ind_start].start)return "";
	while(1)
	{
		if((subtitles[ind_start].start<=stream_pos) && (subtitles[ind_start].end>=stream_pos))
			return subtitles[ind_start].name;
		if(ind_end-ind_start==1)
		{
			if((subtitles[ind_end].start>stream_pos) && (subtitles[ind_start].end<stream_pos))
				return "";
   			return subtitles[ind_end].name;
		}	
		int newpos=(ind_start+ind_end)/2;
		if(subtitles[newpos].start>stream_pos)
			ind_end=newpos;
   		else if(subtitles[newpos].end<stream_pos)
			ind_start=newpos;
		else
    	    	    return subtitles[newpos].name;
	}
}
int AviPlayer::HasSubtitles()
{
    if(subtitles.size()==0)return 0;
    return 1;
}    


void AviPlayer::start()
{
    if(initialized==1)
    {
	cout<<"Can't start(), already started"<<endl;
	return;
    }	
    if(clip==0)
    {
	cout<<"Can't start(), no clip"<<endl;
	return;
    }	
    quit=0;
    if(stream)
        stream->SeekToTime(0);
    if(ar)
    {
	ar->reseek(0);
        ar->start();
    }	
    time_start=0;
    paused=0;
    initialized=1;
}

void AviPlayer::stop()
{
    hangup=1;
    if(ar)
	ar->stop();
    while(hangup)usleep(100000);    
    time_start=0;
    killhandler(0);
}
void AviPlayer::endPlayer()
{
    quit=1;
    if(stream)
    {
	int result=pthread_join(main_thread, NULL);
	if(result==EDEADLK)
	{
	    printf("ERROR ERROR ERROR :  AviPlayer::endPlayer() called from main_thread\n");
	    pthread_exit(NULL);
	}
    }	    
    if(ar)delete ar;
    if(clip)
    {
	cout<<"Closing clip"<<endl;
	delete clip;
    }	
    clip=0;    
    stream=0;
    initialized=0;
    ar=0;
    if(frames_video)
	cout<<"Played "<<frames_video<<" video frames ( " << frame_drop*100./frames_video << "% drop )" << endl;
}

double AviPlayer::reseek(double pos)
{
#ifndef QUIET
    cout<<"Seek pos: "<<pos<<endl;
#endif
    if(initialized==0)return -1;
    hangup=1;
    cout<<"Waiting for main_thread to hang up"<<endl;
    while(hangup)usleep(10000);
    cout<<"OK!\n";
    pos=stream->SeekToTime(pos);
#ifndef QUIET
    cout<<"Keyframe pos: "<<pos<<endl;
#endif
    if(ar)
        ar->reseek(pos);
    if(paused)
    {
        if(stream->ReadFrame()==0)
	    frames_video++;
        (*drawfunc)(stream->GetFrame());
    }	    
    time_start=0;
    initialized=1;
    return pos;
}
int AviPlayer::page_up()
{
    if(initialized==0)return -1;
    if(paused==0)return -1;
    int result=stream->ToNextKeyFrame();
    if(result==-1)return -1;
// We can't predict what happens if main_thread won't 'sleep' during reseek
    hangup=1;
    cout<<"Waiting for main_thread to hang up"<<endl;
    while(hangup)usleep(10000);

    double pos=stream->GetTime();
    if(ar)ar->reseek(pos);
    stream->ReadFrame();
    (*drawfunc)(stream->GetFrame());

    time_start=0;// we must clear time_start after each reseek
    initialized=1;

    return 0;
}    
int AviPlayer::page_down()
{
    if(initialized==0)return -1;
    if(paused==0)return -1;
    int result=stream->ToPrevKeyFrame();
    if(result==-1)return -1;

    hangup=1;
    cout<<"Waiting for main_thread to hang up"<<endl;
    while(hangup)usleep(10000);

    double pos=stream->GetTime();
    if(ar)ar->reseek(pos);
    stream->ReadFrame();
    (*drawfunc)(stream->GetFrame());

    time_start=0;
    initialized=1;

    return 0;
}        
int AviPlayer::reseek_exact(double pos)
{
    double pos2=0;
    if(initialized==0)return -1;
    hangup=1;
    cout<<"Waiting for main_thread to hang up"<<endl;
    while(hangup)usleep(10000);
    cout<<"OK!\n";
    if(stream)
	pos2=stream->SeekToTime(pos);
    if(ar)ar->reseek(pos);
    cout<<"Seek OK\n";
    if(pos2<0)
    {
	if(stream)stream->SeekToTime(0);
	if(ar)ar->reseek(0);
	time_start=0;
	initialized=1;
	cout<<"Returning\n";
	return -1;
    }	
    if(pos2>pos)
    {
	cout<<"ERROR: reseek_exact: pos2>pos"<<endl;
	return -1;
    }	
    if(stream)
    {
	cout<<"Adjusting pos, please wait\n";
	while(stream->GetTime()<pos)
	{
	    if(stream->Eof())
		return -1;
	    if(stream->ReadFrame()==0)
	        frames_video++;
	}	
    }
    time_start=0;
    initialized=1;
    cout<<"Success"<<endl;
    return 0;
}

IAviPlayer* CreateAviPlayer()
{
    IAviPlayer* answer=new AviPlayer;
    return answer;
}    

double GetAvifileVersion()
{
    return AVIFILE_VERSION;
}    
