/********************************************************

	Video decoder interface
	Copyright 2000 Eugene Smith (divx@euro.ru)
	Last modified: 28.07.2000

*********************************************************/


#include <stdio.h>
#include <iostream>
#include <math.h>

#include <loader.h>

#include <wine/vfw.h>
#include <default.h>
#include <except.h>
using namespace std;
/**
 *
 *  Class for video decompression.
 *
 *  Usage:
 *    
 *  Init() call tries to load video codec corresponding to your
 * format, and throws FatalError if something goes wrong. 
 *  Start() starts decompression and Stop() stops it.
 *
 * SetBitDepth() sets desired bit depth of output picture. Returns
 * zero on success, -1 if bit depth is unsupported. Most decoders 
 * support depths 16 ( weight 555 ), 24 or 32.
 *
 * DecodeFrame() decodes a single frame. Returns zero on success, any
 * other value on failure. Nonzero here is not fatal ( maybe it was
 * just one defective bit in file, after next keyframe everything 
 * will be OK ).
 *
 */

struct my_BITMAPINFO
{
    BITMAPINFOHEADER header;
    int colors[3];
    my_BITMAPINFO(){}
    my_BITMAPINFO(const BITMAPINFOHEADER& hdr)
    {
	header=hdr;
	setBitFields();
    }
    void setBitFields()
    {	
	header.biSize=sizeof(BITMAPINFOHEADER)+12;
	header.biCompression=3;//BI_BITFIELDS
	header.biBitCount=16;
	header.biSizeImage=abs(2*header.biWidth*header.biHeight);
	colors[0]=0xF800;
	colors[1]=0x07E0;
	colors[2]=0x001F;
    }	
    void setRGB()
    {
	header.biSize=sizeof(BITMAPINFOHEADER);
	header.biCompression=0;//BI_RGB
    }	
};    
class VideoDecoder
{
public:
    VideoDecoder():m_outFrame(0),m_iState(0){}
    ~VideoDecoder()
    {
	try
	{
    	    if(m_iState==2)Stop();
	    if(m_iState==1)Close();
	}
	catch(FatalError& error)
	{
	    cerr<<"~VideoDecoder(): ";
	    error.Print();
	}    
    }	
    void Init(const char*) throw(FatalError&);//BITMAPINFOHEADER*
    void Close();
    void Start();
    void Stop();
    char* GetFrame()
    {
	return m_outFrame;
    }	
    BITMAPINFOHEADER& DestFmt(){return m_obh.header;}
    int DecodeFrame(char* src, int size, int is_keyframe);
    int SetBitDepth(int bits);
//    int SetExtendedMode(int mode, int size);//mode=mmioFOURCC('Y', 'U', 'Y', '2')
    //size=12 (in bits)
//    int SetColorSpace(int csp);
private:
    HIC m_hic;
    int m_iState;
    BITMAPINFOHEADER m_bh;
    my_BITMAPINFO m_obh;
    char* m_outFrame;
};    
    